/* Codigo base: Commander.H -  Aaron Young, Ryan Neighbour */
/* Sergio Nuñez Covarrubias */

#ifndef SPECIALCOMMANDER_H
#define SPECIALCOMMANDER_H

#include "EventHandler.H"
#include "GameObj.H"
#include "WatcherAux.H"
#include "ModuleSet.H"

class SpecialCommander: public EventHandler{

    bool constructed;           ///< True if constructed
    std::set<GameObj*> units;   ///< Set of owned units
	std::set<SpecialCommander*> comms; ///< Set of sub-commanders
    	
protected:
    /**
     * Pointer to immediate superior.
     * myCommander = 0 for root Commander.
     * @see Commander()
     * @see ~Commander()
     */
    SpecialCommander * myCommander;
	
    /**
     * Easy access point to numerous helper modules.
     */
    ModuleSet & moduleSet;	
	
	/**
     * Count of how many updates until exec() is called.
     * Derived by rand(X)%maxUpdateCount + 1.
     * @see update()
     * @see exec()
     */
    int updateCounter;
	
	/**
     * Commander's id within ORTS.
     */
    int commanderID;
	
	/**
     * Maximum amount of update calls before exec() will be called.
     * Actual number set randomly.
     * @see update()
     * @see exec()
     */
    int maxUpdateCount;

protected:

    /**
     * The main function (or brain) of the commander.
     * Must be implemented in derived Commanders.
     * Will be called only once ever rand(X)%maxUpdateCount + 1 updates.
     * Runs when updateCounter == 0.
     * @see update()
     * @see maxUpdateCount
     * @see updateCounter
     */
    virtual void exec() = 0;
	virtual void onReceiveUnit(GameObj *, Watcher::WatchID) = 0;
    virtual void onLoseUnit(GameObj *) = 0;
    virtual bool onEvent(const Event &) = 0;    
    
public:

	/**
	 * Obtiene el vector units.
	 * @return Vector units.
	 */
	std::set<GameObj*> const getUnits(){
		return units;
	}
	
	/**
	 * Constructor for Commander.
	 * @param moduleSet reference to the global module set.
	 * @param maxUpdate an integer that will be used as the highest possible random number of update() calls before exec() is run.
	 * @see maxUpdateCount
	 * @see updateCounter
	 * @see ~Commander()
	 */  
    SpecialCommander(ModuleSet& moduleSet, int maxUpdate=25);
	

	/**
	 * Constructor for Commander.
	 * @param mySuperior a pointer of type Commander to the Commander's superior.
	 * @see myCommander
	 * @param maxUpdate an integer that will be used as the highest possible random number of update() calls before exec() is run.
	 * @see maxUpdateCount
	 * @see updateCounter
	 * @see ~Commander()
	 */  
    SpecialCommander(SpecialCommander* superior, int maxUpdate=25);
	
	
	/**
	 * Destroys the Commander and all subcommanders recursively.
	 * Sends units to superior Commander.
	 * @see commVector
	 * @see myCommander
	 * @see unitVector
	 * @see Commander()
	 */
    virtual ~SpecialCommander();
	
	
	/**
	 * Decriments updateCounter by 1.
	 * Recursively calls update on all Commanders.
	 * When updateCounter == 0 updateCounter is set to 
	 * a new random number (maxUpdateCount = maximum) and exec() is called.
	 * @see updateCounter
	 * @see maxUpdateCount
	 * @see exec()
	 */
	void update();
    
	
	/**
	 * Manejador de eventos.
	 * @param e Evento ocurrido
	 */
    bool handle_event(const Event &event);
	
	
	/**
	 * Obtiene el ID de commander.
	 * @return commander ID.
	 */
	sint4 getID() const;
    
	
	/**
	 * Añade una determinada unidad. Si es un edificio le aplica seguimiento.
	 * @param unit Unidad a añadir.
	 */
    void addUnit(GameObj * unit);
	
	
	/**
	 * Añade una determinada unidad al vector units.
	 * @param unit Unidad a añadir.
	 */
	void addUnit2(GameObj * unit);
	
	
	/**
	 * Elimina una determinada unidad.
	 * @param unit Unidad a eliminar.
	 */
    void removeUnit(GameObj * unit);


    template <typename CONTAINER>
    void addUnits(const CONTAINER & cont){
        typename CONTAINER::const_iterator it;
        for(it = cont.begin(); it != cont.end(); it++){
            addUnit( (*it) );
        }
    }

    template <typename CONTAINER>
    void removeUnits(const CONTAINER & cont){
        typename CONTAINER::const_iterator it;
        for(it = cont.begin(); it != cont.end(); it++){
            removeUnit( (*it) );
        }
    }
    
    /// Will return the maximum number of units that will
    /// be provided in response to a call to demandUnits()
    /// @param bp_name the type of unit to be counted
    /// @return the maximum number of units demandUnits() will provide
    /// @see demandUnits
    size_t countOwned(const std::string & bp_name) const;
};

#endif
