/* Codigo base: RouteAllocator.C */
/* Sergio Nuñez Covarrubias */

#include "SpecialRouteAllocator.H"
#include "SpecialRoute.H"
#include <algorithm>

#define TEST(x)

using namespace std;

///////////////////////////////CONSTRUCTORS/DESTRUCTORS/////////////////////////////

SpecialRouteAllocator::SpecialRouteAllocator() : workers(0), pos(0), idPos(0){ }

SpecialRouteAllocator::SpecialRouteAllocator(sint4 workers_) : workers(workers_), pos(0), idPos(0){ }

SpecialRouteAllocator::~SpecialRouteAllocator(){ }


/**
 * create a new unique ID to assign to a new route
 * @return the ID number
 */
sint4 SpecialRouteAllocator::newID() {
    idPos++;
    return idPos;
}


/**
 * adds a route to this allocator.
 * @param route the pointer to a route
 */
void SpecialRouteAllocator::addRoute(RoutePtr route_){
        route_->setID(newID());
        routes.push_back(route_);
}


/**
 * increments the number of workers who could use the allocator.
 */
void SpecialRouteAllocator::addWorker(){
    workers++;
}


/**
 * returns the best available route, bearing in mind the location of the unit
 */
RoutePtr SpecialRouteAllocator::getBestRoute(TerrainBase::Loc locationOfWorker, GameObj* cc, boost::shared_ptr<MineralCluster> cluster){

   
	RoutePtr smallestRoute;
    real8 smallestDist = 10000;


    FORALL(routes, rit){

		if ((*rit)->getCurrentWorkers() >= (*rit)->getMaxWorkers()){			
            continue;
        }

		if(!((cc == (*rit)->getCC()) &&  (esMineralCluster((*rit)->getMineralRec(), cluster)))){			
			continue;
		}		

        real8 distance = (*rit)->getAccessPoint().distance(locationOfWorker);
        if (distance < smallestDist){
	
            smallestDist = distance;
            smallestRoute = *rit;
        }		
    }
	return smallestRoute;
} 

/**
 * Comprueba si el mineral pertenece al cluster.
 * @return true si el mineral pertenece al cluster, false en caso contrario.
 */
bool SpecialRouteAllocator::esMineralCluster(const History::Record* mineral, boost::shared_ptr<MineralCluster> cluster){
	bool esMC = false;
	
	std::set<const History::Record *> myMinerals;
	cluster->getMinerals(myMinerals);

	std::set<const History::Record *>::iterator it;
	
	for (it = myMinerals.begin(); it != myMinerals.end(); it++){
		
		if(esMC)
			break;
						
		if((*it) == mineral){
			esMC = true;			
		}	
	}
	return esMC;
}


/**
 * returns the total number of routes
 */
sint4 SpecialRouteAllocator::getNumRoutes(){
    return routes.size();
}


/**
 * returns the number of available routes
 */
uint4 SpecialRouteAllocator::getNumAvailableRoutes(){
    sint4 count = 0;
    FORALL(routes, rit){
        if ((*rit)->getCurrentWorkers() < (*rit)->getMaxWorkers()){
            count++;
        }
    }
    return count;
}


void SpecialRouteAllocator::markRouteAsBad(RoutePtr route){
    route->planFailureOccurred();
    if(route->getPlanFailures() > 10){
        FORALL_DYN(routes, rit){
            if (route == *rit){
                routes.erase(rit);
                break;
            }
        }
    }
}


/**
 * returns the worst route currently being used (for taking out a unit)
 * @return a pointer to the route.
 */
RoutePtr SpecialRouteAllocator::getWorstUsedRoute(){
    RoutePtr worstRoute;
    Vector<RoutePtr>::reverse_iterator rit;
    for(rit = routes.rbegin(); rit != routes.rend(); rit++){
        if ((*rit)->getCurrentWorkers() > 0){
            worstRoute = *rit;
        }
    }
    return worstRoute;
}


/**
 * removes a route from this allocator
 * @param route the route to remove
 */
void SpecialRouteAllocator::removeRoute(RoutePtr route_){
    FORALL_DYN(routes, rit){
        if (*rit == route_){
            routes.erase(rit);
            break;
        }
    }
}


/**
 * decrements the number of workers using the allocator
 */
void SpecialRouteAllocator::removeWorker(){
    workers++;
}


/**
 * searches through the routes and erases any invalid route lenghts (ie -1, 0)
 */
void SpecialRouteAllocator::removeInaccessibleRoutes(){
    Vector<RoutePtr>::iterator rit = routes.begin();
    while (rit != routes.end()){
        if ((*rit)->getRouteLength() < 1){            
            rit = routes.erase(rit);
        }else{
            rit++;
        }
    }
}


/**
 * sets the number of workers using this allocator
 * @param numWorkers the number of workers
 */
void SpecialRouteAllocator::setWorkers(sint4 numWorkers){
    workers = numWorkers;
}


/**
 * sorts the routes in ascending order of distance from a location.
 */
void SpecialRouteAllocator::sortRoutesAccordingToLocation(){
    removeInaccessibleRoutes();
    sort(routes.begin(), routes.end(), CompareRoutesByDistanceToLocation());
}


/**
 * Determine the number of workers needed (in total) to use every
 * available route.
 * @return the number of workers needed
 */
size_t SpecialRouteAllocator::countWorkersNeeded() const{
    size_t count = 0;
    FORALL(routes, rit){
        count += (*rit)->getMaxWorkers();
    }
    return count;
}