/* Codigo base: Worker.C */
/* Sergio Nuñez Covarrubias */

#include "SpecialWorker.H"
#include "SpecialRoute.H"
#include "PlayerInfo.H"
#include "GameStateModule.H"
#include "Game.H"
#include "GameObj.H"

const std::string miningComponent = "pickaxe";
const std::string miningAction = "mine";
const std::string returnResourcesAction= "return_resources";

/////////////////////////////// Constructors ///////////////////////////
SpecialWorker::SpecialWorker(GameObj* gob_,SimpleGather *commander_,GameStateModule *gsm_)
    : gsm(gsm_), commander(commander_),	state(IDLE), knowsRoute(false){
		gob = gob_;
}

SpecialWorker::~SpecialWorker(){ 
    if(route != 0){
        route->removeWorker();
    }
}

//////////////////////////////////////PUBLIC///////////////////////////////////////

/**
 * sets knowsRoute to false
 */
void SpecialWorker::clearRoute(){
    if(route != 0){
        route->removeWorker();
        route.reset();
    }	
}


/**
 * whether the worker is close enough to drop off its minerals at the control
 * centre.
 * @return whether it's close enough
 */
bool SpecialWorker::closeEnoughToDropOff() const{
    assert(route != 0);
    sint4 dist = (sint4)(Game::distance(*gob, *route->getCC()));
    return dist <= WORKER_CC_DISTANCE_THRESHOLD;
}


/**
 * determines if the worker is close enough to mine and moves if necessary
 * @return whether it's close enough
 */
bool SpecialWorker::closeEnoughToMine() const {
    assert(route != 0);

    if (route->mineralIsActive()){
        sint4 dist = (sint4)(Game::distance(*gob, *(route->getMineralRec()->getObject())));
        if (dist > WORKER_MINERAL_DISTANCE_THRESHOLD){
            commander->moveTo(gob, route->getMineralRec()->getObject());
            return false;
        }
        else
            return true;
    }
    return false;
}


/**
 * sends the action for a worker to drop its minerals at the control centre
 */
void SpecialWorker::dropOffMinerals(){
    assert(route != 0);

    PlayerInfo& pi = gsm->get_game().get_cplayer_info();
    Vector<sint4> args(1, pi.get_id(route->getCC()));
    gob->set_action(returnResourcesAction, args);
    setState(IDLE);
}


/**
 * the control center that this worker returns to
 * @return a pointer to the control centre game object
 */
GameObj* SpecialWorker::getControlCenter() const{
    assert(route != 0);
    return route->getCC();
}


/**
 * the amount of minerals this worker is carrying
 * @return amount of minerals
 */
sint4 SpecialWorker::getCurrentMinerals() const{ 
    return gob->get_int("minerals"); 
}


/**
 * the game object of this worker
 * @return a pointer to the game object
 */
GameObj* SpecialWorker::get_gob() const{
    return gob;
}


/**
 * the CURRENT location of this worker
 * @return the location
 */
TerrainBase::Loc SpecialWorker::getLocation() const{
    return TerrainBase::Loc(*(gob->sod.x),*(gob->sod.y)); 
}


/**
 * the mineral that this worker mines from
 * @return the history record of the mineral
 */
const History::Record* SpecialWorker::getMineralRec() const{ 
    if(route != 0){
        return route->getMineralRec(); 
    }else{
        return NULL;
    }
}


/**
 * The route that this worker is using
 * @return a pointer to the route
 */
RoutePtr SpecialWorker::getRoute() const{
    // PRECONDITION //
    assert( route != 0 );
    return route;
}


/**
 * the current state of this worker
 * @return IDLE, MOVING, RETURNING, MINING
 */
sint4 SpecialWorker::getState() const{
    return state;
}


/**
 * whether this worker has an allocated route
 */
bool SpecialWorker::hasRoute() const{    
	return route != 0;
}


/**
 * whether this worker is currently mining
 */
bool SpecialWorker::isMining() const{
    return state == MINING;
}


/**
 * sends the action to mine from the mineral
 */
void SpecialWorker::mine(){
    assert(route != 0);

    setState(MINING);
    PlayerInfo& pi = gsm->get_game().get_cplayer_info();
    Vector<sint4> args(1, pi.get_id(route->getMineralRec()->getObject()));
    gob->component(miningComponent)->set_action(miningAction, args);
}


/**
 * moves this worker to its control center
 */
void SpecialWorker::moveToCC(){
    assert(route != 0);

    route->moveToCC(gob);
    setState(RETURNING);
}


/**
 * moves this worker to its mineral
 */
void SpecialWorker::moveToMineral(){
    assert(route != 0);

    setState(MOVING);
    route->moveToMineral(gob);
}


/**
 * sets the route that this worker should use
 */
void SpecialWorker::setRoute(RoutePtr route){
    assert(route != NULL);

    if(this->route != 0){
        this->route->removeWorker();
    }

    this->route = route;
    this->route->addWorker();	
}


/**
 * set the state of this worker.
 * @param state can be IDLE, MOVING, MINING or RETURNING
 */
void SpecialWorker::setState(sint4 state_){
    state = state_;
}
